/******************************************************************************
 FILE:		LoadDspFile.c

 INPUT:		pci_fd  The reference to the PCI board as returned by "open()".
 		filename  The name of the file to load. Typically called "pci.rom".
 			 
 DESCRIPTION:	This function downloads an ascii PCI ROM file into the DSP
 		ROM space (R).

 AUTHOR:	Scott Streit
 DATE:		02/01/2000
 
 REVISION HISTORY:
 ----------------
 02/01/2000	sds	Initial

******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "DSPCommand.h"
#include "PCIBurn.h"

int load_rom_file(int pci_fd, const char *filename)
{
	const char *input_value = " ";
	char input_line[81];
	const char *type_string = "__";
	const char *addr_string = " ";
	const char *data_string = " ";
	const char *word_count_string = " ";
	const char *file_type = " ";
	const char *path = "./";
	const char *name = " ";
	int end_of_file = 0;
	int done_reading_data = 0;
	int type = 0;
	int addr = 0;
	int data = 0;
	int word_count = 0;
	int n = -1;
	long file_position = 0;
	FILE *inFile;
	int reply_value = 0;
	int count = 0;
	int i = 0;
	int j = 0;
	const char *tokens[20];

	/* Open the file for reading. */
	if ((inFile = fopen(filename, "r")) == NULL) {
		printf("\nError: Cannot open file: %s \n", filename);
		exit(EXIT_FAILURE);
	}

	/* Check that first data value < 4096 (0x1000). And check */
	/* that the second data value equals zero. Reset the file */
	/* pointer back to the start of the line when finished.   */
	fgets(input_line, 80, inFile);
	fgets(input_line, 80, inFile);
	fgets(input_line, 80, inFile);
	fgets(input_line, 80, inFile);
			
	word_count_string = strtok(input_line, " ");
	addr_string = strtok(NULL, " ");
			
	sscanf(word_count_string, "%X", &word_count);
	sscanf(addr_string, "%X", &addr);
			
	if (word_count > 0x1000) {
		printf("\nError: Word count out of DSP memory range.\n");
		exit(1);
	}
			
	if (addr != 0) {
		printf("\nError: Address not equal to zero.\n");
		exit(1);
	}
	addr = 0;
	rewind(inFile);
	
	/* Read in the file one line at a time.	 */
	while (end_of_file == 0) {
		fgets(input_line, 80, inFile);
		
		input_value = strtok(input_line, " ");
		
		/* Check for the start of valid data. */
		if (strcmp(input_value, "_DATA") == 0) {
			type_string = strtok(NULL, " ");
			addr_string = strtok(NULL, " ");

			/* Convert the "address" string to the correct data type. */
			sscanf(addr_string, "%X", &addr);
				
			/* Read in the data block. */
			while (!done_reading_data) {
				fgets(input_line, 80, inFile);
	
				count = 0;
				tokens[count] = strtok(input_line, " ");
					
				if (strstr(tokens[0], "_") != NULL) {
					done_reading_data = 1;
					fseek (inFile, -15, SEEK_CUR);
				}
				
				else {
					while (tokens[count] != NULL) {
						count++;
						tokens[count] = strtok(NULL, " ");
					}
									
					for (i=0; i<count-1; i++) {
						sscanf(tokens[i], "%X", &data);
						printf("addr = 0x%06X  data = 0x%06X", addr, data);
						reply_value = write_memory(pci_fd, addr, data);
						addr++;
						
						/* Display the addr and data in the same place on the screen */
						for (j=0; j<32; j++)
							printf("\b");
						
						/* Check for proper reply */
						if (reply_value != DON) {
							fclose(inFile);
							return EXIT_FAILURE;
						}
					}
				}
			}
		}
		
		/* Check the file validity and set the board destination. */
		if (strcmp(input_value, "_START") == 0) {
			file_type = strtok(NULL, " ");
			
			if (strcmp(file_type, "PCIBOOT") == 0) {
				printf("\nROM file ... OK. \n\n");
				set_board_destination(pci_fd, WRM_ARG_NUM, PCI_ID);
				set_type(R);
			}
	
			else {
				printf("\n Error Invalid file type.");
				exit(EXIT_FAILURE);
			}
		}
		
		/* Check for the end of file. */
		else if (strcmp(input_value, "_END") == 0) {
			end_of_file = 1;
		}
						
		/* Re-initialize variables. */
		done_reading_data = 0;
		type_string = "__";
		input_value = NULL;
	}
		
	fclose(inFile);
	
	return EXIT_SUCCESS;
}
