/******************************************************************************
 FILE:		PciRomBurn.c

 INPUT:		None
 			 
 DESCRIPTION:	This program opens a PCI device driver and burns the PCI ROM
 		with a ROM file (pci.rom).

 AUTHOR:	Scott Streit
 DATE:		02/01/2000
 
 REVISION HISTORY:
 ----------------
 02/01/2000	sds	Initial
 02/22/2000	sds	Updated to check that the total number of words to write
 			is less than 4096 and the start address equals 0. Also
 			modified the output statements. Removed the reply buffer
 			status bit check.
 04/26/2000	sds	Removed the CLEAR_REPLY_BUFFER ioctl() commands ... they
 			are no longer supported by the driver.
 08/15/2000	sds	Removed PCI download option.
******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include <stropts.h>
#include "PciRomBurn.h"
#include "DSPCommand.h"
#include "PCIBurn.h"

#define PCI_DEV		"/dev/astropci0\0"
#define DEBUG		0
#define READY		1
#define NOT_READY	0

int main()
{
	int pci_fd;
	int reply_value;
	int check;
	char pci_filename[80];
	char rom_filename[80];
	char proceed = '\n';
	char do_pci = '\n';
	char do_rom = '\n';
	time_t start_time = time(NULL);

	/* Initialize the filenames */
	strcpy(pci_filename, "");
	strcpy(rom_filename, "");
	
	/* Try to open a device driver connection. */
 	if ((pci_fd=open(PCI_DEV, O_RDWR)) == -1) {
    		perror("\n\nError: Could not open device driver!\n");
    		exit(EXIT_FAILURE);
    	}

	/* Inform user of consequences. */
	printf("\n****** WARNING  WARNING  WARNING  WARNING ******");
	printf("\nYou are about to overwrite the contents of your PCI board ROM.");
	printf("\nYou will not be able to recover the current version without");
	printf("\nthe original. A new ROM will need to be obtained if this does");
	printf("\nnot work. You must have used the \"rom\" script to generate a");
	printf("\npci.rom file.");
	
	while (1) {
		printf("\nProceed (Y/N): ");
		proceed = getchar();
		if (proceed == 'Y' || proceed == 'y' || proceed == 'N' || proceed == 'n')
			break;

		fflush(stdin);
	}
	
	/* Function getchar() leaves the \n behind, so flush it. */
	fflush(stdin);
	
	if (proceed == 'n' || proceed == 'N') {
		printf("\nNo harm done ... bye!\n\n");
		exit(EXIT_SUCCESS);
	}
	
	else if (proceed == 'y' || proceed == 'Y') {
		while (1) {
			printf("\nPermanently Write PCI ROM File (Y/N): ");
			#ifdef LINUX
				do_rom = getchar();	/* Remove extra characters from stdin. */
			#endif
			do_rom = getchar();
			if (do_rom == 'Y' || do_rom == 'y' || do_rom == 'N' || do_rom == 'n')
				break;
			
			/* Flush stdin to prevent multiple "Permanently Write PCI ROM File (Y/N):" statements */
			/* from being output when multiple characters are entered as input.  		      */
			fflush(stdin);
		}
		
		/* Function getchar() leaves the \n behind, so flush it. */
		fflush(stdin);
		
		if (do_rom == 'y' || do_rom == 'Y') {
			while (strcmp(rom_filename,"") == 0) {
				printf("\nEnter PCI ROM filename (*.rom): ");
				scanf("%s", &rom_filename[0]);
			}
		}

		if (do_rom == 'y' || do_rom == 'Y') {
			printf("\nWriting the ROM file... \n");
			check = load_rom_file(pci_fd, rom_filename);
	    		
	    		if (check == EXIT_SUCCESS) {
	    			printf("\n\nPCI ROM file successfully written.\n");
				printf("\n****** NOTE  NOTE  NOTE  NOTE  NOTE  NOTE ******");
				printf("\nTo complete the process, you must power off and on");
				printf("\nyour computer. This will load the new ROM contents");
				printf("\ninto the DSP.\n");
			}
		}

	}

	else
		printf("\nSorry, not a valid response ... bye!\n\n");
	
	/* Close the PCI board connection. */
	close(pci_fd);

	printf("\nROM Burn Finished!\n");

	return 0;
}

